#include <stdio.h>
#include "/opt/vidpb/lib/avilib.h"

#define MAX_FILES 255
#define OURBUFSIZ 4096000
char buf[OURBUFSIZ];

void die(char *msg) {
  fprintf(stderr,msg);
  fprintf(stderr,"\n");
  exit(1);
}


int main(int ac,char **av) {
  avi_t *inh[MAX_FILES],*outh;
  long frames,width,height,audiochannels;
  long audiobits,audioformat,audiorate,audiobytes,audiosamps,audiobps;
  double framerate;
  char *compressor;
  long fr,i,frlen,left,now;
  long totalaudiobytes=0,totalvideoframes=0;

  if (ac<=2) {
    die("usage: avijoin OUTFILE INFILE1 INFILE2 INFILE3...");
  }

  if ((outh=AVI_open_output_file(av[1]))==NULL) {
    die("Can't write output avi file");
  }

  for(i=2;i<ac;i++) {
    if ((inh[i]=AVI_open_input_file(av[i],1))==NULL) {
      die("Can't read avi file");
    }

    if (i==2) {
      width=AVI_video_width(inh[i]);
      height=AVI_video_height(inh[i]);
      framerate=AVI_frame_rate(inh[i]);
      compressor=AVI_video_compressor(inh[i]);
      audiochannels=AVI_audio_channels(inh[i]);
      audiobits=AVI_audio_bits(inh[i]);
      audioformat=AVI_audio_format(inh[i]);
      audiorate=AVI_audio_rate(inh[i]);
    }
  }
  AVI_set_video(outh,width,height,framerate,compressor);
  AVI_set_audio(outh,audiochannels,audiorate,audiobits,audioformat);

  for(i=2;i<ac;i++) {
    frames=AVI_video_frames(inh[i]);
    audiobytes=AVI_audio_bytes(inh[i]);
    printf("Processing video: %s (%d frames)\n",av[i],frames);

    for(fr=0;fr<frames;fr++) {
      frlen=AVI_read_frame(inh[i],buf);
      AVI_write_frame(outh,buf,frlen);
    }

    printf("Processing audio: %s\n",av[i]);
    AVI_set_audio_position(inh[i],0);
    left=audiobytes;
    while(left>0) {
      now=(left>OURBUFSIZ) ? OURBUFSIZ : left;
      AVI_read_audio(inh[i],buf,now);
      AVI_write_audio(outh,buf,now);
      left-=now;
      printf("%cread %d this time; audio left %d        ",13,now,left); 
      fflush(stdout);
    }
    printf("\n");
    AVI_close(inh[i]);
  }
  AVI_close(outh);
}



#if 0
  char *infile=av[0],*outfile=av[1];
  long int framestart=atoi(av[2]);
  long int frameend=atoi(av[3]);
  avi_t *inh,*outh;
  long frames,width,height,audiochannels;
  long audiobits,audioformat,audiorate,audiobytes,audiosamps,audiobps;
  double framerate;
  char *compressor;
  char buf[OURBUFSIZ];

  setvbuf(stdout,NULL,_IONBF,0);
  strcpy(infile,av[1]);
  strcpy(outfile,av[2]);
  printf("reading %s\n",infile);

  if ((inh=AVI_open_input_file(infile,1))==NULL) {
    die("Can't read avi file");
  }
  if ((outh=AVI_open_output_file(outfile))==NULL) {
    die("Can't write avi file");
  }
  width=AVI_video_width(inh);
  height=AVI_video_height(inh);
  framerate=AVI_frame_rate(inh);
  compressor=AVI_video_compressor(inh);
  audiochannels=AVI_audio_channels(inh);
  audiobits=AVI_audio_bits(inh);
  audioformat=AVI_audio_format(inh);
  audiorate=AVI_audio_rate(inh);
  frames=AVI_video_frames(inh);
  audiobytes=AVI_audio_bytes(inh);
  audiorate=44100;

  AVI_set_video(outh,width,height,framerate,compressor);
  AVI_set_audio(outh,audiochannels,audiorate,audiobits,audioformat);

  AVI_seek_start(inh);

  /* copy the video */
  printf("copy video, %d frames\n",frames);
  for(i=0;i<frames;i++) {
    length=AVI_read_frame(inh,buf);
    if (framectr%UPDATE_RATE==0) {
      printf("%cvideo %d length %d        ",13,i,length); 
    }
    AVI_write_frame(outh,buf,length);
    if ((framectr++)%13!=0) {
#ifdef USE_DUP
      AVI_write_frame(outh,buf,length); 
#else
      AVI_dup_frame(outh);
#endif
    }
  }
  printf("\n\n"); 

#ifdef DO_AUDIO
  /* copy the audio */
  printf("copy audio, %d bytes\n",audiobytes);
  AVI_set_audio_position(inh,0);
  left=audiobytes;
  while(left>0) {
    now=(left>OURBUFSIZ) ? OURBUFSIZ : left;
    AVI_read_audio(inh,buf,now);
    AVI_write_audio(outh,buf,now);
    left-=now;
    printf("%cread %d this time; audio left %d        ",13,now,left); 
  }
  printf("\n\n");
#endif

  AVI_close(inh);
  AVI_close(outh);
}


#endif
