#include <stdio.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <jpeglib.h>
#include "avilib.h"
#include "dvavilib.h"
#include <libdv/dv.h>
#include <gdk-pixbuf/gdk-pixbuf.h>


/* frame size should *always* be big enough to handle largest frame */
#define MAXFRAMESIZE 4000000 
#define MAX_ROWS 1536
#define BITSPERSAMPLE 24
#define BYTESPERSAMPLE (BITSPERSAMPLE/8)
char inbuf[MAXFRAMESIZE];
char workbuf[MAXFRAMESIZE];
JOCTET outbuf[MAXFRAMESIZE];
static struct jpeg_compress_struct cinfo;
static struct jpeg_error_mgr jerr;
static struct jpeg_destination_mgr dest;
static dvavi_t *inh;
static avi_t *outh;
static dv_decoder_t *decoder; 


void die(char *msg) {
  fprintf(stderr,msg);
  fprintf(stderr,"\n");
  exit(1);
}


/* called by the jpeg lib; not to be used by mjpeg api caller */
void mjpeg_init_destination(j_compress_ptr cinfo) {
  cinfo->dest->next_output_byte=outbuf;
  cinfo->dest->free_in_buffer=MAXFRAMESIZE;
}

/* called by the jpeg lib; not to be used by mjpeg api caller */
boolean mjpeg_empty_output_buffer(j_compress_ptr cinfo) {
  /* this should never occur! */
  die("empty_output_buffer was called!");
  return(FALSE);
}

/* called by the jpeg lib; not to be used by mjpeg api caller */
void mjpeg_term_destination(j_compress_ptr cinfo) {
  AVI_write_frame(outh,outbuf,MAXFRAMESIZE-(cinfo->dest->free_in_buffer));
}



int main(int ac,char **av) {
  char *infile=av[1],*outfile=av[2];
  long frames,width,height,audiochannels;
  long audiobits,audioformat,audiorate,audiobytes,audiosamps,audiobps;
  double fps;
  char *compressor;
  int i,j;
  dv_aaux_as_t *dv_aaux_as;
  int samples_this_frame,fd;
  char ch;
  FILE *aufp;
  long now,left;


  decoder = dv_decoder_new();
  dv_init();
  decoder->quality = DV_QUALITY_BEST;


  printf("reading %s\n",infile);

  if ((inh=DVAVI_open_input_file(infile,1))==NULL) {
    die("Can't read avi file");
  }
  if ((outh=AVI_open_output_file(outfile))==NULL) {
    die("Can't write avi file");
  }

  width=DVAVI_video_width(inh);
  height=DVAVI_video_height(inh);

#if defined(PAL)
  width=384;
  height=288;
#elif defined(NTSC)
  width=352;         
  height=240;
#else
#error Please define either NTSC or PAL
#endif

  fps=DVAVI_frame_rate(inh);
  compressor=DVAVI_video_compressor(inh);
  audiochannels=DVAVI_audio_channels(inh);
  audiobits=DVAVI_audio_bits(inh);
  audioformat=DVAVI_audio_format(inh);
  audiorate=DVAVI_audio_rate(inh);
  frames=DVAVI_video_frames(inh);
  audiobytes=DVAVI_audio_bytes(inh);

  printf("Video setup: width=%d, height=%d, fps=%f\n",
		width,height,fps);
  printf("audio setup: channels=%d, rate=%d, bits=%d, format=%d\n",
		audiochannels,audiorate,audiobits,audioformat);
  AVI_set_video(outh, width, height, fps, "MJPG");
  AVI_set_audio(outh, audiochannels, audiorate, audiobits, audioformat);

  /* copy the video */
  DVAVI_seek_start(inh);
  printf("Copying %d frames\n",frames);
  for(i=0;i<frames;i++) { 
    JSAMPROW row_pointer[MAX_ROWS];
    int bread,bwritten;

    printf("%cframe %d         ",13,i);
    fflush(stdout);
    bread=DVAVI_read_frame(inh,inbuf);

	/* convert DV frame to RGB */
	{
    	guchar *pixels[3];
    	guint16 pitches[3];

    	pixels[0] = (guchar*)workbuf;
    	pixels[1] = NULL;
    	pixels[2] = NULL;

    	pitches[0] = inh->width * 3;
    	pitches[1] = 0;
    	pitches[2] = 0;

        if (dv_parse_header(decoder, inbuf)<0) {
          die("Failed to parse dv image header");
        }
    	dv_decode_full_frame(decoder, inbuf, e_dv_color_rgb, pixels, pitches);
	}


	/* do we need to resize the image first? */
	{
        GdkPixbuf *destpix,*srcpix;
        guchar *pixels;

        srcpix=gdk_pixbuf_new(GDK_COLORSPACE_RGB,FALSE,BITSPERSAMPLE/3,
            inh->width, inh->height);
        pixels=gdk_pixbuf_get_pixels(srcpix);
        memcpy(pixels,workbuf,(inh->width*BITSPERSAMPLE/8)*inh->height);
        destpix=gdk_pixbuf_scale_simple(srcpix, width,height,GDK_INTERP_HYPER);
        pixels=gdk_pixbuf_get_pixels(destpix);
        memcpy(workbuf,pixels,(width*BITSPERSAMPLE/8)*height);
        gdk_pixbuf_unref(destpix);
        gdk_pixbuf_unref(srcpix);
    }


    /* create jpeg object */
    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_compress(&cinfo);
  
    /* initialize object */
    cinfo.image_width=width;
    cinfo.image_height=height;
    cinfo.input_components=3;
    cinfo.in_color_space=JCS_RGB;
    jpeg_set_defaults(&cinfo);
  
  
    /* create destination manager */
    dest.init_destination=mjpeg_init_destination;
    dest.empty_output_buffer=mjpeg_empty_output_buffer;
    dest.term_destination=mjpeg_term_destination;
    cinfo.dest=&dest;
 
    /* let's go! */
    jpeg_start_compress(&cinfo, TRUE);
    for(j=0;j<cinfo.image_height;j++) {
      row_pointer[j]=(char *)(workbuf+(j*cinfo.image_width*BITSPERSAMPLE/8));
    }
    bwritten=jpeg_write_scanlines(&cinfo,row_pointer,cinfo.image_height);

    if (bwritten!=cinfo.image_height) {
      printf("only wrote %d!\n",bwritten);
      die("Failed to write complete jpeg frame");
    }
  
    jpeg_finish_compress(&cinfo);
    jpeg_destroy_compress(&cinfo);
  }


  printf("\nProcessing audio\n");
  DVAVI_set_audio_position(inh,0);
  left=audiobytes;
  while(left>0) {
    now=(left>MAXFRAMESIZE) ? MAXFRAMESIZE : left;
    DVAVI_read_audio(inh,workbuf,now);
    AVI_write_audio(outh,workbuf,now);
    left-=now;
    printf("%cread %d this time; audio left %d        ",13,now,left);
    fflush(stdout);
  }
  AVI_close(outh);
  chmod(outfile,0666);
}
