/*
 * avi.h library for AVI file format i/o
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _AVI_H
#define _AVI_H 1

#define QUADWORD signed long long
#define DWORD signed long
#define LONG unsigned long
#define WORD signed short
#define BYTE unsigned char
#define FOURCC unsigned long

#define RIFF_NO_PARENT (-1)
#define RIFF_LISTSIZE (4)
#define RIFF_HEADERSIZE (8)


#define AVI_INDEX_OF_INDEXES (0x00)
#define AVI_INDEX_OF_CHUNKS (0x01)
#define AVI_INDEX_2FIELD (0x01)

enum { AVI_PAL, AVI_NTSC, AVI_AUDIO_48KHZ, AVI_AUDIO_44KHZ, AVI_AUDIO_32KHZ };

typedef struct
{
        DWORD   dwMicroSecPerFrame;             // frame display rate (or 0L)
        DWORD   dwMaxBytesPerSec;               // max. transfer rate
        DWORD   dwPaddingGranularity;           // pad to multiples of this
        // size; normally 2K.
        DWORD   dwFlags;                        // the ever-present flags
        DWORD   dwTotalFrames;                          // # frames in file
        DWORD   dwInitialFrames;
        DWORD   dwStreams;
        DWORD   dwSuggestedBufferSize;

        DWORD   dwWidth;
        DWORD   dwHeight;

        DWORD   dwReserved[4];
}
MainAVIHeader;

typedef struct
{
        WORD    top, bottom, left, right;
}
RECT;

typedef struct
{
        FOURCC  fccType;
        FOURCC  fccHandler;
        DWORD   dwFlags;                /* Contains AVITF_* flags */
        WORD    wPriority;
        WORD    wLanguage;
        DWORD   dwInitialFrames;
        DWORD   dwScale;
        DWORD   dwRate;                 /* dwRate / dwScale == samples/second */
        DWORD   dwStart;
        DWORD   dwLength;               /* In units above... */
        DWORD   dwSuggestedBufferSize;
        DWORD   dwQuality;
        DWORD   dwSampleSize;
        RECT    rcFrame;
}
AVIStreamHeader;

typedef struct
{
        DWORD   dwDVAAuxSrc;
        DWORD   dwDVAAuxCtl;
        DWORD   dwDVAAuxSrc1;
        DWORD   dwDVAAuxCtl1;
        DWORD   dwDVVAuxSrc;
        DWORD   dwDVVAuxCtl;
        DWORD   dwDVReserved[2];
}
DVINFO;

typedef struct
{
        DWORD   dwSize;
        LONG    biWidth;
        LONG    biHeight;
        WORD    biPlanes;
        WORD    biBitCount;
        DWORD   biCompression;
        DWORD   biSizeImage;
        LONG    biXPelsPerMeter;
        LONG    biYPelsPerMeter;
        DWORD   biClrUsed;
        DWORD   biClrImportant;
}
BITMAPINFOHEADER;

typedef struct
{
        WORD    wFormatTag;
        WORD    nChannels;
        DWORD   nSamplesPerSec;
        DWORD   nAvgBytesPerSec;
        WORD    nBlockAlign;
        WORD    wBitsPerSample;
        //WORD  cbSize;
}
WAVEFORMATEX;

typedef struct
{
        WORD    wLongsPerEntry;
        BYTE    bIndexSubType;
        BYTE    bIndexType;
        DWORD   nEntriesInUse;
        DWORD   dwChunkId;
        DWORD   dwReserved[3];
        struct avisuperindex_entry
        {
                QUADWORD        qwOffset;
                DWORD   dwSize;
                DWORD   dwDuration;
        }
        aIndex[2014];
}
AVISuperIndex;

typedef struct
{
        WORD    wLongsPerEntry;
        BYTE    bIndexSubType;
        BYTE    bIndexType;
        DWORD   nEntriesInUse;
        DWORD   dwChunkId;
        QUADWORD        qwBaseOffset;
        DWORD   dwReserved;
        struct avifieldindex_entry
        {
                DWORD   dwOffset;
                DWORD   dwSize;
        }
        aIndex[4028];
}
AVIStdIndex;

typedef struct
{
        DWORD   dirEntryType;
        DWORD   dirEntryName;
        DWORD   dirEntryLength;
        size_t  dirEntryOffset;
        int     dirEntryWrittenFlag;
        int     dirEntryParentList;
}
AviDirEntry;


#endif


