#ifndef DVAVILIB_H
#define DVAVILIB_H


typedef struct
{
   long pos;
   long len;
} dvvideo_index_entry;

typedef struct
{
   long pos;
   long len;
   long tot;
} dvaudio_index_entry;

typedef struct
{
   long   fdes;              /* File descriptor of DVAVI file */
   long   mode;              /* 0 for reading, 1 for writing */

   long   width;             /* Width  of a video frame */
   long   height;            /* Height of a video frame */
   double fps;               /* Frames per second */
   char   compressor[8];     /* Type of compressor, 4 bytes + padding for 0 byte */
   long   video_strn;        /* Video stream number */
   long   video_frames;      /* Number of video frames */
   char   video_tag[4];      /* Tag of video data */
   long   video_pos;         /* Number of next frame to be read
                                (if index present) */

   long   a_fmt;             /* Audio format, see #defines below */
   long   a_chans;           /* Audio channels, 0 for no audio */
   long   a_rate;            /* Rate in Hz */
   long   a_bits;            /* bits per audio sample */
   long   audio_strn;        /* Audio stream number */
   long   audio_bytes;       /* Total number of bytes of audio data */
   long   audio_chunks;      /* Chunks of audio data in the file */
   char   audio_tag[4];      /* Tag of audio data */
   long   audio_posc;        /* Audio position: chunk */
   long   audio_posb;        /* Audio position: byte within chunk */

   long   pos;               /* position in file */
   long   n_idx;             /* number of index entries actually filled */
   long   max_idx;           /* number of index entries actually allocated */
   unsigned char (*idx)[16]; /* index entries (DVAVI idx1 tag) */
   dvvideo_index_entry * video_index;
   dvaudio_index_entry * audio_index;
   long   last_pos;          /* Position of last frame written */
   long   last_len;          /* Length of last frame written */
   int    must_use_index;    /* Flag if frames are duplicated */
   long   movi_start;
} dvavi_t;

#define DVAVI_MODE_WRITE  0
#define DVAVI_MODE_READ   1

/* The error codes delivered by avi_open_input_file */

#define DVAVI_ERR_SIZELIM      1     /* The write of the data would exceed
                                      the maximum size of the DVAVI file.
                                      This is more a warning than an error
                                      since the file may be closed safely */

#define DVAVI_ERR_OPEN         2     /* Error opening the DVAVI file - wrong path
                                      name or file nor readable/writable */

#define DVAVI_ERR_READ         3     /* Error reading from DVAVI File */

#define DVAVI_ERR_WRITE        4     /* Error writing to DVAVI File,
                                      disk full ??? */

#define DVAVI_ERR_WRITE_INDEX  5     /* Could not write index to DVAVI file
                                      during close, file may still be
                                      usable */

#define DVAVI_ERR_CLOSE        6     /* Could not write header to DVAVI file
                                      or not truncate the file during close,
                                      file is most probably corrupted */

#define DVAVI_ERR_NOT_PERM     7     /* Operation not permitted:
                                      trying to read from a file open
                                      for writing or vice versa */

#define DVAVI_ERR_NO_MEM       8     /* malloc failed */

#define DVAVI_ERR_NO_DVAVI       9     /* Not an DVAVI file */

#define DVAVI_ERR_NO_HDRL     10     /* DVAVI file has no has no header list,
                                      corrupted ??? */

#define DVAVI_ERR_NO_MOVI     11     /* DVAVI file has no has no MOVI list,
                                      corrupted ??? */

#define DVAVI_ERR_NO_VIDS     12     /* DVAVI file contains no video data */

#define DVAVI_ERR_NO_IDX      13     /* The file has been opened with
                                      getIndex==0, but an operation has been
                                      performed that needs an index */

/* Possible Audio formats */

#define WAVE_FORMAT_UNKNOWN             (0x0000)
#define WAVE_FORMAT_PCM                 (0x0001)
#define WAVE_FORMAT_ADPCM               (0x0002)
#define WAVE_FORMAT_IBM_CVSD            (0x0005)
#define WAVE_FORMAT_ALAW                (0x0006)
#define WAVE_FORMAT_MULAW               (0x0007)
#define WAVE_FORMAT_OKI_ADPCM           (0x0010)
#define WAVE_FORMAT_DVI_ADPCM           (0x0011)
#define WAVE_FORMAT_DIGISTD             (0x0015)
#define WAVE_FORMAT_DIGIFIX             (0x0016)
#define WAVE_FORMAT_YAMAHA_ADPCM        (0x0020)
#define WAVE_FORMAT_DSP_TRUESPEECH      (0x0022)
#define WAVE_FORMAT_GSM610              (0x0031)
#define IBM_FORMAT_MULAW                (0x0101)
#define IBM_FORMAT_ALAW                 (0x0102)
#define IBM_FORMAT_ADPCM                (0x0103)


dvavi_t* DVAVI_open_output_file(char * filename);
void DVAVI_set_video(dvavi_t *DVAVI, int width, int height, double fps, char *compressor);
void DVAVI_set_audio(dvavi_t *DVAVI, int channels, long rate, int bits, int format);
int  DVAVI_write_frame(dvavi_t *DVAVI, char *data, long bytes);
int  DVAVI_dup_frame(dvavi_t *DVAVI);
int  DVAVI_write_audio(dvavi_t *DVAVI, char *data, long bytes);
long DVAVI_bytes_remain(dvavi_t *DVAVI);
int  DVAVI_close(dvavi_t *DVAVI);

dvavi_t *DVAVI_open_input_file(char *filename, int getIndex);

long DVAVI_video_frames(dvavi_t *DVAVI);
int  DVAVI_video_width(dvavi_t *DVAVI);
int  DVAVI_video_height(dvavi_t *DVAVI);
double DVAVI_frame_rate(dvavi_t *DVAVI);
char* DVAVI_video_compressor(dvavi_t *DVAVI);

int  DVAVI_audio_channels(dvavi_t *DVAVI);
int  DVAVI_audio_bits(dvavi_t *DVAVI);
int  DVAVI_audio_format(dvavi_t *DVAVI);
long DVAVI_audio_rate(dvavi_t *DVAVI);
long DVAVI_audio_bytes(dvavi_t *DVAVI);

long DVAVI_frame_size(dvavi_t *DVAVI, long frame);
int  DVAVI_seek_start(dvavi_t *DVAVI);
int  DVAVI_set_video_position(dvavi_t *DVAVI, long frame);
long DVAVI_read_frame(dvavi_t *DVAVI, char *vidbuf);
int  DVAVI_set_audio_position(dvavi_t *DVAVI, long byte);
long DVAVI_read_audio(dvavi_t *DVAVI, char *audbuf, long bytes);

int  DVAVI_read_data(dvavi_t *DVAVI, char *vidbuf, long max_vidbuf,
                               char *audbuf, long max_audbuf,
                               long *len);

void DVAVI_print_error(char *str);
char *DVAVI_strerror();
char *DVAVI_syserror();

#endif
